
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <GL/glfw.h> // http://www.glfw.org/GLFWReference27.pdf

#ifndef M_PI
#define M_PI 3.141592654
#endif

int running=1;

//-------------- textures --------------
typedef unsigned char byte;

#pragma pack(push)
#pragma pack(1)

typedef struct
{
    byte  identsize;          // size of ID field that follows 18 byte header (0 usually)
    byte  colourmaptype;      // type of colour map 0=none, 1=has palette
    byte  imagetype;          // type of image 0=none,1=indexed,2=rgb,3=grey,+8=rle packed

    short colourmapstart;     // first colour map entry in palette
    short colourmaplength;    // number of colours in palette
    byte  colourmapbits;      // number of bits per palette entry 15,16,24,32

    short xstart;             // image x origin
    short ystart;             // image y origin
    short width;              // image width in pixels
    short height;             // image height in pixels
    byte  bpp;               // image bits per pixel 8,16,24,32
    byte  descriptor;         // image descriptor bits (vh flip bits)

    // pixel data follows header

} TGA_HEADER;

#pragma pack(pop)

GLuint loadTGA(const char* fname)
{
	FILE* f=fopen(fname,"rb");
	if (!f)
    {
        fprintf(stderr,"Error opening TGAfile %s\n",fname);
        return 0;
    }

	TGA_HEADER head;
	fread(&head,1,sizeof(TGA_HEADER),f);
	GLvoid* pixels=0;
	GLuint texID=0;
	if (head.imagetype==2) // RGB not packed
	{
	    pixels=(unsigned char*)malloc(head.width*head.height*head.bpp >> 3);
	    fread(pixels,head.height,head.width*(head.bpp>>3),f);
	    glGenTextures(1,&texID);
	    glBindTexture(GL_TEXTURE_2D,texID);
	    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);	// Linear Filtering
	    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);	// Linear Filtering

        switch (head.bpp)
        {
		    case 24 :
			    glTexImage2D(GL_TEXTURE_2D,0,GL_RGB8,head.width,head.height,0,GL_BGR_EXT,GL_UNSIGNED_BYTE,pixels);
		    break;
            case 32 :
			    glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA8,head.width,head.height,0,GL_RGBA,GL_UNSIGNED_BYTE,pixels);
		    break;
        }
	}
	free(pixels);
	fclose(f);
	return texID;
}

GLuint gColorMapID=0;


void onInit()
{
	gColorMapID=loadTGA("./textures/rockwall_color.tga");
}

/* handle rendering */
void onRender()
{
	glDisable(GL_CULL_FACE);
    glEnable(GL_DEPTH_TEST);

	glClearColor(0.0,0.1,0.4,1.0);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);


	glBindTexture(GL_TEXTURE_2D, gColorMapID);
    glEnable(GL_TEXTURE_2D);

  glMatrixMode( GL_MODELVIEW );
  glLoadIdentity();
  glTranslatef( 0.0, 0.0, -2.0 );


    glColor4f(1.0,1.0,1.0,1.0);
	glBegin(GL_QUADS);

		glTexCoord2f(0.0, 0.0);
		glVertex2f(-1.0, -1.0);

		glTexCoord2f(1.0, 0.0);
		glVertex2f(1.0, -1.0);

		glTexCoord2f(1.0, 1.0);
		glVertex2f(1.0, 1.0);

		glTexCoord2f(0.0, 1.0);
		glVertex2f(-1.0, 1.0);

	glEnd();
}


/* new window size callback */
void GLFWCALL onReshape( int width, int height )
{
  GLfloat h = (GLfloat) height / (GLfloat) width;
  GLfloat xmax, znear, zfar;

  znear = 0.1f;
  zfar  = 30.0f;
  xmax  = znear * 1.0f;

  glViewport( 0, 0, (GLint) width, (GLint) height );
  glMatrixMode( GL_PROJECTION );
  glLoadIdentity();
  glFrustum( -xmax, xmax, -xmax*h, xmax*h, znear, zfar );
}

/* handle keyboard input */
void GLFWCALL onKey( int k, int action )
{
  if( action != GLFW_PRESS ) return;

  switch (k) {
    case GLFW_KEY_ESC:
        running = 0;
    break;
    default:
        return;
  }
}

/* handle mouse move event */
void GLFWCALL onMouseMove( int x, int y )
{
    int button = GLFW_MOUSE_BUTTON_LEFT;
 /*
 GLFW_MOUSE_BUTTON_LEFT
 GLFW_MOUSE_BUTTON_RIGHT
 GLFW_MOUSE_BUTTON_MIDDLE
 */
    int state = glfwGetMouseButton( button );

    if (state == GLFW_PRESS)
    {
    // process
    } 
}

/* program entry */
int main(int argc, char *argv[])
{
    if( !glfwInit() )
    {
        fprintf( stderr, "Failed to initialize GLFW\n" );
        exit( EXIT_FAILURE );
    }
/*
glfwOpenWindow(int width, int height,
    int redbits, int greenbits, int bluebits, int alphabits,
    int depthbits, int stencilbits,
    int mode)
*/
    if( !glfwOpenWindow( 512,512, 0,0,0,0, 16,0, GLFW_WINDOW ) )
    {
        fprintf( stderr, "Failed to open GLFW window\n" );
        glfwTerminate();
        exit( EXIT_FAILURE );
    }

    glfwSetWindowTitle( "03_texturing" );
    glfwEnable( GLFW_KEY_REPEAT );
    glfwSwapInterval( 1 ); // VSync 1x

    // Set callback functions
    glfwSetWindowSizeCallback( onReshape );

    glfwSetKeyCallback( onKey );
    glfwSetMousePosCallback( onMouseMove );

    onInit();

    // Main loop
    while( running )
    {
         onRender();
        // Swap buffers
        glfwSwapBuffers();
        glfwSleep( 0.01 );    // don't eat whole CPU time
        // Was the window closed?
        if( !glfwGetWindowParam( GLFW_OPENED ) )
        {
            running = 0;
        }
    }

    // Terminate GLFW
    glfwTerminate();

    // Exit program
    exit( EXIT_SUCCESS );
}

